/***************************************************
 * Autor: Ingmar Stapel
 * Website: www.custom-build-robots.com
 * Date: 2020-04-21
 * Description:
 * This program will enable you robot car to be controlled
 * via WIFI and a small HTML control center.
 ****************************************************/
 
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

// The servo controller PCA9685 is connected to the I2C bus.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

// Set the WIFI SSID and password
// SSID of your WIFI
const char* ssid = "XXXXXXXXX"; 
//password of your WIFI
const char* password = "XXXXXXXX"; 

// Set the default speed for the left and right motor
// the value is between 0 and 99 like percent.
int speed_left = 0;
int speed_right = 0;

// Start the web server right now
ESP8266WebServer server(80);

void setup() {
  // set the serial speed
  Serial.begin(9600);
  
  // set the PCA9685 servo controller
  pwm.begin();
  pwm.setOscillatorFrequency(27000000);
  pwm.setPWMFreq(1600);  
  Wire.setClock(400000);

  // connect to your local WIFI
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  // connecting to wifi
  WiFi.begin(ssid, password); 

  // while wifi not connected
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    //print "...."
    Serial.print("."); 
  }
  Serial.println("");
  Serial.println("WiFi connected");
  // Print the IP address 
  Serial.println(WiFi.localIP());  

  server.on("/", handleRoot);
  server.on("/car", handleCar);

  server.on("/inline", []() {
    server.send(200, "text/plain", "this works as well");
  });

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("HTTP server started");
}

void handleRoot() {
  //server.send(200, "text/plain", "hello from esp8266!");
    String message = "";
    message += "<html> <head> <title>ESP8266 NodeMCU robot car</title><head>";
    message += "<body><h3>Control your robot car</h1>";
    message += "<p> <a href=\"/car\"> ---> START YOUR ENGINE <--- </a></p>";
    message += "</body></html>";
    server.send(200, "text/html", message);
 }

// set for each motor (left or right) the spinning direction
void set_motor_mode(String motor, String motor_direction)
{
  if(motor == "left")
  {
    if(motor_direction == "forward")
    {
        pwm.setPWM(1, 0, 4096 );
        pwm.setPWM(2, 4096, 0 );
        Serial.println("left | forward");      
    }
    else if (motor_direction == "backward")
    {
        pwm.setPWM(1, 4096, 0 );
        pwm.setPWM(2, 0, 4096 );    
        Serial.println("left | backward");   
    }
  }

  if(motor == "right")
  {
    if(motor_direction == "forward")
    {
        pwm.setPWM(4, 0, 4096 );
        pwm.setPWM(3, 4096, 0 );   
        Serial.println("right | forward");    
    }
    else if (motor_direction == "backward")
    {
        pwm.setPWM(4, 4096, 0 );
        pwm.setPWM(3, 0, 4096 );  
        Serial.println("right | backward");     
    }    
  }
}

// handle each request and detect which button was pressed.
void handleCar() {
 String message = "";
 int BtnValue = 0;
  for (uint8_t i = 0; i < server.args(); i++) {
    if (server.argName(i)=="a")
    {
      String s = server.arg(i);
      BtnValue = s.toInt();
    }
    Serial.println(server.argName(i) + ": " + server.arg(i) + "\n");
  }

  switch (BtnValue) {
   case 1: //do nothing, the speed of the left motor will be displayed.
      break;
   case 2: // accelerate forward
      
      speed_left += 20;
      speed_right += 20;

      // check that speed_left is not higher then 100%
      if (speed_left >= 100)
      {
        speed_left = 99;
      }
      // check that speed_right is not higher then 100%
      if (speed_right >= 100)
      {
        speed_right = 99;
      }

      // set the turning direction and speed of the left motor
      if (speed_left >=0)
      {
        set_motor_mode("left", "forward");
        pwm.setPWM(0, 4096 - (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_left)));
      }
      else if (speed_left < 0)
      {
        set_motor_mode("left", "backward");
        pwm.setPWM(0, 4096 + (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_left)));
      }
      
      // set the turning direction and speed of the right motor
      if (speed_right >=0)
      {
        set_motor_mode("right", "forward");
        pwm.setPWM(5, 4096 - (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_right)));
      }
      else if (speed_right < 0)
      {
        set_motor_mode("right", "backward");
        pwm.setPWM(5, 4096 + (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_right)));
      }      
      break;
  case  3:// do nothing, the speed of the right motor will be displayed.
     break;
  case  4:// turn left 
      speed_left += 20;
      speed_right -= 20;

      // check that speed_left is not higher then 100%
      if (speed_left >= 100)
      {
        speed_left = 99;
      }
      // check that speed_right is not higher then 100%
      if (speed_right <= -100)
      {
        speed_right = -99;
      }

      // set the turning direction and speed of the left motor
      if (speed_left >=0)
      {
        set_motor_mode("left", "forward");
        pwm.setPWM(0, 4096 - (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_left)));
      }
      else if (speed_left < 0)
      {
        set_motor_mode("left", "backward");
        pwm.setPWM(0, 4096 + (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_left)));
      }

      // set the turning direction and speed of the right motor
      if (speed_right >=0)
      {
        set_motor_mode("right", "forward");
        pwm.setPWM(5, 4096 - (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_right)));
      }
      else if (speed_right < 0)
      {
        set_motor_mode("right", "backward");
        pwm.setPWM(5, 4096 + (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_right)));
      }                
     break;
  case 5: // stop the left and right motor 
   speed_left = 0;
   speed_right = 0;
   // stop the left motor
   pwm.setPWM(0, 0, 4096 );
   // stop the right motor
   pwm.setPWM(5, 0, 4096 );
   Serial.println("PWM right: " + String(4096 + (40.96 * speed_left)));
   Serial.println("PWM right: " + String(4096 + (40.96 * speed_right)));
   break;     
  case  6://   
      speed_left -= 20;
      speed_right += 20;

      // check that speed_left is not higher then 100%
      if (speed_left <= -100)
      {
        speed_left = -99;
      }
      // check that speed_right is not higher then 100%
      if (speed_right >= 100)
      {
        speed_right = 99;
      }

      // set the turning direction and speed of the left motor
      if (speed_left >=0)
      {
        set_motor_mode("left", "forward");
        pwm.setPWM(0, 4096 - (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_left)));
      }
      else if (speed_left < 0)
      {
        set_motor_mode("left", "backward");
        pwm.setPWM(0, 4096 + (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_left)));
      }
      
      // set the turning direction and speed of the right motor
      if (speed_right >=0)
      {
        set_motor_mode("right", "forward");
        pwm.setPWM(5, 4096 - (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_right)));
      }
      else if (speed_right < 0)
      {
        set_motor_mode("right", "backward");
        pwm.setPWM(5, 4096 + (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_right)));
      }      
    break;
  case 7:// do nothing and display nothing
    break;
  case 8:// // accelerate backward
      speed_left -= 20;
      speed_right -= 20;

      // check that speed_left is not higher then 100%
      if (speed_left <= -100)
      {
        speed_left = -99;
      }
      // check that speed_right is not higher then 100%
      if (speed_right <= -100)
      {
        speed_right = -99;
      }

      // set the turning direction and speed of the left motor
      if (speed_left >=0)
      {
        set_motor_mode("left", "forward");
        pwm.setPWM(0, 4096 - (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_left)));
      }
      else if (speed_left < 0)
      {
        set_motor_mode("left", "backward");
        pwm.setPWM(0, 4096 + (40.96 * speed_left), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_left)));
      }
      
      // set the turning direction and speed of the right motor
      if (speed_right >=0)
      {
        set_motor_mode("right", "forward");
        pwm.setPWM(5, 4096 - (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 - (40.96 * speed_right)));
      }
      else if (speed_right < 0)
      {
        set_motor_mode("right", "backward");
        pwm.setPWM(5, 4096 + (40.96 * speed_right), 0 );
        Serial.println("PWM right: " + String(4096 + (40.96 * speed_right)));
      }      
      break;    
  case 9:// do nothing and display nothing
    break;
  }

  // now create the HTML string which will be displayed as control center
 
  message += "<html> <head> <title>ESP8266 NodeMCU robot car</title><head>";
  message += "<body><h3>Control your robot car</h1>";
  message += "<table> "; 
  message += "<tr>";
  message += "<td><p><a href=\"/car?a=1\"><button style=\"width:250;height:250;font-size:40px;\" class=\"button\">speed left</br>";
  message += speed_left;
  message += "</button></a></p> ";
  message += "<td><p><a href=\"/car?a=2\"><button style=\"width:250;height:250;font-size:180px;\" class=\"button\">^</button></a></p> ";
  message += "<td><p><a href=\"/car?a=3\"><button style=\"width:250;height:250;font-size:40px;\" class=\"button\">speed right</br>";
  message += speed_right;
  message += "</button></a></p> ";
  message += "<tr>";
  message += "<td><p><a href=\"/car?a=4\"><button style=\"width:250;height:250;font-size:180px;\" class=\"button\"> < </button></a></p> ";
  message += "<td><p><a href=\"/car?a=5\"><button style=\"width:250;height:250;font-size:100px;\" class=\"button\">Stop</button></a></p> ";
  message += "<td><p><a href=\"/car?a=6\"><button style=\"width:250;height:250;font-size:180px;\" class=\"button\"> > </button></a></p> ";
  message += "<tr>";
  message += "<td><p><a href=\"/car?a=7\"><button style=\"width:250;height:250;font-size:100px;\" class=\"button\"></button></a></p> ";
  message += "<td><p><a href=\"/car?a=8\"><button style=\"width:250;height:250;font-size:180px;\" class=\"button\">v</button></a></p> ";
  message += "<td><p><a href=\"/car?a=9\"><button style=\"width:250;height:250;font-size:100px;\" class=\"button\"></button></a></p> ";
  message += "</table> "; 
  message += "</body></html>";
  server.send(200, "text/html", message);
 }

// if something went wrong....
void handleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}
 
void loop() {
  server.handleClient();
}
